@@ -50,14 +50,17 @@ std::vector<std::vector<double>>  bayes(   std::vector<double> y,
                                            std::vector<double> lambda, 
                                            std::vector<double> pi, 
                                            std::vector<double> gamma, 
-                                           double vg, 
                                            double vb, 
+                                           double vg, 
                                            double ve,
                                            double ssb_prior,
+                                           double ssg_prior,
                                            double sse_prior,
                                            double nub,
+                                           double nug,
                                            double nue,
                                            bool updateB,
+                                           bool updateG,
                                            bool updateE,
                                            bool updatePi,
                                            int nit,
@@ -72,13 +75,11 @@ std::vector<std::vector<double>>  bayes(   std::vector<double> y,
   int nc = pi.size();
   double nsamples=0.0;
   
-  double rhs, lhs, bn, conv, diff, mu;
+  double rhs, lhs, bn, diff, mu;
   double rhs0, rhs1, lhs0, lhs1, like0, like1, p0;
   double ssb, ssg, sse, dfb, dfe, dfg, chi2;
-  double ssg_prior, nug;
   double lambda_tau, mu_tau, u, lambda2;
-  //double shape, shape0, rate, rate0, lambda2;
-  
+
   std::vector<int> d(m);
   
   std::vector<double> e(n),g(n);
@@ -86,23 +87,11 @@ std::vector<std::vector<double>>  bayes(   std::vector<double> y,
   std::vector<int> order(m), mask(m);
   
   std::vector<double> dm(m),bm(m),vbi(m),vei(m);
-  //std::vector<double> ves(nit),vbs(nit),vgs(nit),pis(nit),mus(nit);
   std::vector<double> ves(nit+nburn),vbs(nit+nburn),vgs(nit+nburn),pis(nit+nburn),mus(nit+nburn);
   
-  std::vector<double> vbscale(nc), pic(nc), pim(nc), probc(nc), logLc(nc);
+  std::vector<double> pim(nc), probc(nc), logLc(nc);
   double cumprobc, vbc, v0, v1, logLcAdj;
   
-  for ( int i = 0; i < nc; i++) {
-    vbscale[i]=gamma[i];
-    pic[i]=pi[i];
-  }
-  
-  // Mean adjust y and initialize e
-  mu = std::accumulate(y.begin(), y.end(), 0.0)/n;
-  for ( int i = 0; i < n; i++) {
-    e[i] = y[i] - mu;
-  }                
-
   std::fill(g.begin(), g.end(), 0.0);
   std::fill(wy.begin(), wy.end(), 0.0);
   std::fill(ww.begin(), ww.end(), 0.0);
@@ -115,6 +104,12 @@ std::vector<std::vector<double>>  bayes(   std::vector<double> y,
   std::fill(pim.begin(), pim.end(), 0.0);
   
 
+  // Mean adjust y and initialize e
+  mu = std::accumulate(y.begin(), y.end(), 0.0)/n;
+  for ( int i = 0; i < n; i++) {
+    e[i] = y[i] - mu;
+  }                
+  
   // Initialize variables
   for (int i = 0; i < m; i++) {
     for (int j = 0; j < n; j++) {
@@ -126,9 +121,6 @@ std::vector<std::vector<double>>  bayes(   std::vector<double> y,
     vbi[i]=vb;
   }
   
-  nug=nub;
-  ssg_prior=((nug-2.0)/nug)*vg;
-  
   lambda2 = m;
   
   // Establish order of markers as they are entered into the model
@@ -140,16 +132,10 @@ std::vector<std::vector<double>>  bayes(   std::vector<double> y,
   
   // Start Gibbs sampler
   std::random_device rd;
-  //unsigned int local_seed;
-  //local_seed = rd();
-  //std::mt19937 gen(local_seed);
   std::mt19937 gen(seed);
-  
-  
 
   for ( int it = 0; it < nit+nburn; it++) {
-    conv = 0.0;
-
+    
     if ( (it > nburn) && (it % nthin == 0) ) {
       nsamples = nsamples + 1.0;
     }
@@ -242,7 +228,6 @@ std::vector<std::vector<double>>  bayes(   std::vector<double> y,
         for (int j = 0; j < n; j++) {
           e[j]=e[j] - W[i][j]*(diff);
         }
-        conv = conv + diff*diff;
         b[i] = bn;
       }
     }
@@ -279,7 +264,6 @@ std::vector<std::vector<double>>  bayes(   std::vector<double> y,
             e[j]=e[j] - W[i][j]*(diff);
           }
         }
-        conv = conv + diff*diff;
         b[i] = bn;
       }
 
@@ -361,7 +345,6 @@ std::vector<std::vector<double>>  bayes(   std::vector<double> y,
             e[j]=e[j] - W[i][j]*(diff);
           }
         }
-        conv = conv + diff*diff;
         b[i] = bn;
       }
       // Sample pi for Bayes R
@@ -386,7 +369,6 @@ std::vector<std::vector<double>>  bayes(   std::vector<double> y,
     }
     
     
-
     // Sample marker variance
     ssb = 0.0;
     dfb = 0.0;
@@ -434,17 +416,6 @@ std::vector<std::vector<double>>  bayes(   std::vector<double> y,
       for ( int j = 0; j < n; j++) {
         sse = sse + e[j]*e[j];
       }
-      // if(method==3) {
-      //   // Park & Casella - Campos et al.
-      //   ssb=0.0;
-      //   dfb=0.0;
-      //   for ( int i = 0; i < m; i++) {
-      //     ssb = ssb + b[i]*(1.0/lambda[i])*b[i]/ve;
-      //     dfb = dfb + 1.0;
-      //   }
-      //   sse=sse+ssb;
-      //   dfe=dfe+dfb;
-      // }
       std::chi_squared_distribution<double> rchisq(dfe);
       chi2 = rchisq(gen);
       ve = (sse + sse_prior*nue)/chi2 ;
@@ -559,303 +530,38 @@ std::vector<std::vector<double>>  bayes(   std::vector<double> y,
 
 
 
-
 // [[Rcpp::export]]
-std::vector<std::vector<double>>  sbayes( std::vector<double> wy,
-                                          std::vector<std::vector<double>> LD,
-                                          std::vector<double> b,
+std::vector<std::vector<double>>  sbayes( double yy,
+                                          std::vector<double>& wy,
+                                          std::vector<double>& ww, 
+                                          std::vector<std::vector<double>>& LDvalues, 
+                                          std::vector<std::vector<int>>& LDindices, 
+                                          std::vector<double> b, 
                                           std::vector<double> lambda,
-                                          double yy,
-                                          double pi,
-                                          double vg,
-                                          double vb,
+                                          std::vector<bool> mask, 
+                                          std::vector<double> pi, 
+                                          std::vector<double> gamma, 
+                                          double vg, 
+                                          double vb, 
                                           double ve,
                                           double ssb_prior,
+                                          double ssg_prior,
                                           double sse_prior,
                                           double nub,
+                                          double nug,
                                           double nue,
                                           bool updateB,
+                                          bool updateG,
                                           bool updateE,
                                           bool updatePi,
-                                          int n,
+                                          bool adjustE,
+                                          int n, 
                                           int nit,
-                                          int method) {
-  
-  // Define local variables
-  int m = b.size();
-  
-  double rhs, lhs, bn, conv, diff;
-  double rhs0, rhs1, lhs0, lhs1, like0, like1, p0;
-  double ssb, sse, ssg, dfb, dfe, chi2;
-  double xtau, tau, lambda_tau, mu_tau, z, z2, u;
-  
-  std::vector<int> d(m);
-  
-  std::vector<double> ww(m),r(m);
-  
-  std::vector<double> dm(m),bm(m);
-  std::vector<double> ves(nit),vbs(nit),pis(nit);
-  
-  std::vector<double> x2(m);
-  std::vector<int> order(m);
-  
-  // Initialize variables
-  for ( int i = 0; i < m; i++) {
-    dm[i] = 0.0;
-    bm[i] = 0.0;
-    ww[i] = LD[i][i];
-    r[i] = wy[i];
-    x2[i] = (wy[i]/ww[i])*(wy[i]/ww[i]);
-  }
-  
-  for (int i=0; i < nit; i++) {
-    vbs[i] = 0.0;
-    ves[i] = 0.0;
-    pis[i] = 0.0;
-  }
-  
-  // Establish order of markers as they are entered into the model
-  std::iota(order.begin(), order.end(), 0);
-  std::sort(  std::begin(order),
-              std::end(order),
-              [&](int i1, int i2) { return x2[i1] > x2[i2]; } );
-  
-  // Wy - W'Wb
-  for ( int i = 0; i < m; i++) {
-    if (b[i]!= 0.0) {
-      for (int j = 0; j < m; j++) {
-        r[j]=r[j] - LD[i][j]*b[i];
-      }
-    }
-  }
-  
-  // Start Gibbs sampler
-  std::random_device rd;
-  unsigned int local_seed;
-  local_seed = rd();
-  std::mt19937 gen(local_seed);
-  
-  for ( int it = 0; it < nit; it++) {
-    conv = 0.0;
-    
-    // Compute marker effects (BLUP)
-    if (method==0) {
-      for ( int isort = 0; isort < m; isort++) {
-        int i = order[isort];
-        lhs = ww[i] + lambda[i];
-        rhs = r[i] + ww[i]*b[i];
-        bn = rhs/lhs;
-        diff = bn-b[i];
-        for (int j = 0; j < m; j++) {
-          r[j]=r[j] - LD[i][j]*(diff);
-        }
-        conv = conv + diff*diff;
-        b[i] = bn;
-      }
-    }
-    
-    // Sample marker effects (Mixed, BayesA, Lasso)
-    if ( method==1 || method==2 || method==3 ) {
-      for ( int isort = 0; isort < m; isort++) {
-        int i = order[isort];
-        lhs = ww[i] + lambda[i];
-        rhs = r[i] + ww[i]*b[i];
-        std::normal_distribution<double> rnorm(rhs/lhs, sqrt(ve/lhs));
-        bn = rnorm(gen);
-        diff = bn-b[i];
-        for (int j = 0; j < m; j++) {
-          r[j]=r[j] - LD[i][j]*(diff);
-        }
-        conv = conv + diff*diff;
-        b[i] = bn;
-      }
-    }
-    
-    // Sample marker effects (BayesC)
-    if (method==4) {
-      for ( int isort = 0; isort < m; isort++) {
-        int i = order[isort];
-        lhs0 = 1.0/vb;
-        lhs1 = ww[i]/ve + 1.0/vb;
-        rhs0 = 0.0;
-        rhs1 = 0.0;
-        rhs1 = r[i]/ve + ww[i]*b[i]/ve;
-        like0 = sqrt((1.0/lhs0))*std::exp(0.5*(1.0/lhs0)*rhs0*rhs0);
-        like1 = sqrt((1.0/lhs1))*std::exp(0.5*(1.0/lhs1)*rhs1*rhs1);
-        like0 = like0*(1.0-pi);
-        like1 = like1*pi;
-        p0 = like0/(like0+like1);
-        d[i]=0;
-        std::uniform_real_distribution<double> runif(0.0, 1.0);
-        u = runif(gen);
-        if(u>p0) d[i]=1;
-        bn=0.0;
-        if(d[i]==1) {
-          std::normal_distribution<double> rnorm(rhs1/lhs1, sqrt(1.0/lhs1));
-          bn = rnorm(gen);
-        }
-        diff = bn-b[i];
-        if(diff!=0.0) {
-          for (int j = 0; j < m; j++) {
-            r[j]=r[j] - LD[i][j]*(diff);
-          }
-          conv = conv + diff*diff;
-        }
-        b[i] = bn;
-      }
-    }
-    
-    // Sample marker variance
-    ssb = 0.0;
-    dfb = 0.0;
-    for ( int i = 0; i < m; i++) {
-      bm[i] = bm[i] + b[i];
-      if(d[i]==1)   {
-        ssb = ssb + b[i]*b[i];
-        dfb = dfb + 1.0;
-        dm[i] = dm[i] + 1.0;
-      }
-    }
-    if(updateB) {
-      std::chi_squared_distribution<double> rchisq(dfb+nub);
-      chi2 = rchisq(gen);
-      vb = (ssb + ssb_prior*nub)/chi2 ;
-      vbs[it] = vb; 
-    }
-    
-    
-    // Sample residual variance
-    if(updateE) {
-      dfe = (double)n + nue;
-      ssg = 0.0;
-      sse = 0.0;
-      for ( int i = 0; i < m; i++) {
-        ssg = ssg + b[i] * (wy[i] -  r[i]);
-        sse = sse + b[i] * (r[i] + wy[i]);
-      }
-      sse = yy - sse;
-      std::chi_squared_distribution<double> rchisq(dfe);
-      chi2 = rchisq(gen);
-      ve = (sse + sse_prior*nue)/chi2 ;
-      ves[it] = ve;
-    }
-    
-    // Update lambda's for BLUP/Mixed
-    if ( method==1) {
-      for ( int i = 0; i < m; i++) {
-        lambda[i] = ve/vb;
-      }
-    }
-    
-    // Sample marker specific variance for Bayes A
-    if (method==2) {
-      dfb = 1.0 + nub;
-      for ( int i = 0; i < m; i++) {
-        ssb = b[i]*b[i];
-        std::chi_squared_distribution<double> rchisq(dfb);
-        chi2 = rchisq(gen);
-        vb = (ssb + ssb_prior*nub)/chi2 ;
-        lambda[i] = ve/vb;
-      }
-    }
-    
-    // Sample marker specific tau for Bayesian lasso
-    if (method==3) {
-      lambda_tau = 2.0*2.0*0.5*0.5/vb;
-      for ( int i = 0; i < m; i++) {
-        ssb = b[i]*b[i];
-        mu_tau = sqrt(lambda_tau/ssb);
-        std::normal_distribution<double> norm(0.0, 1.0);
-        z = norm(gen);
-        z2=z*z;
-        xtau=mu_tau+0.5*mu_tau*mu_tau*z2/lambda_tau - 0.5*(mu_tau/lambda_tau)*sqrt(4*mu_tau*lambda_tau*z2+mu_tau*mu_tau*z2*z2);
-        std::uniform_real_distribution<double> runif(0.0, 1.0);
-        u = runif(gen);
-        tau = mu_tau*mu_tau/xtau;
-        if(u <= mu_tau/(mu_tau+xtau)) tau=xtau;
-        lambda[i] = ve/tau;
-      }
-    }
-    
-    // Sample pi for Bayes C
-    if(method==4 && updatePi) {
-      double count = dfb + 1.0;
-      std::gamma_distribution<double> rgamma(count,1.0);
-      double rg = rgamma(gen);
-      pi = rg/(double)m;
-      pis[it] = pi;
-    }
-  }
-  
-  // Summarize results
-  std::vector<std::vector<double>> result(10);
-  result[0].resize(m);
-  result[1].resize(m);
-  result[2].resize(nit);
-  result[3].resize(nit);
-  result[4].resize(nit);
-  result[5].resize(nit);
-  result[6].resize(m);
-  result[7].resize(m);
-  result[8].resize(3);
-  result[9].resize(m);
-  
-  for (int i=0; i < m; i++) {
-    result[0][i] = bm[i]/nit;
-    result[1][i] = dm[i]/nit;
-  }
-  for (int i=0; i < nit; i++) {
-    //result[2][i] = mus[i];
-    result[2][i] = 0.0;
-    result[3][i] = vbs[i];
-    result[4][i] = ves[i];
-    result[5][i] = pis[i];
-  }
-  for (int i=0; i < m; i++) {
-    result[6][i] = wy[i];
-    result[7][i] = r[i];
-  }
-  result[8][0] = vb;
-  result[8][1] = ve;
-  result[8][2] = pi;
-  for (int i=0; i < m; i++) {
-    result[9][i] = b[i];
-  }
-  
-  
-  return result;
-}
-
-// [[Rcpp::export]]
-std::vector<std::vector<double>>  sbayes_spa( std::vector<double> wy,
-                                              std::vector<double> ww, 
-                                              std::vector<std::vector<double>> LDvalues, 
-                                              std::vector<std::vector<int>> LDindices, 
-                                              std::vector<double> b, 
-                                              std::vector<double> lambda,
-                                              std::vector<bool> mask, 
-                                              double yy, 
-                                              std::vector<double> pi, 
-                                              std::vector<double> gamma, 
-                                              double vg, 
-                                              double vb, 
-                                              double ve,
-                                              double ssb_prior,
-                                              double sse_prior,
-                                              double nub,
-                                              double nue,
-                                              bool updateB,
-                                              bool updateE,
-                                              bool updatePi,
-                                              bool updateG,
-                                              bool adjustE,
-                                              int n, 
-                                              int nit,
-                                              int nburn,
-                                              int nthin,
-                                              int method,
-                                              int algo,
-                                              int seed) {
+                                          int nburn,
+                                          int nthin,
+                                          int method,
+                                          int algo,
+                                          int seed) {
   
   
     
@@ -952,9 +658,6 @@ std::vector<std::vector<double>>  sbayes_spa( std::vector<double> wy,
 
   // Start Gibbs sampler
   std::random_device rd;
-  //unsigned int local_seed;
-  //local_seed = rd();
-  //std::mt19937 gen(local_seed);
   std::mt19937 gen(seed);
   
   for ( int it = 0; it < nit+nburn; it++) {
@@ -1377,14 +1080,14 @@ std::vector<std::vector<double>>  sbayes_spa( std::vector<double> wy,
 
 
 // [[Rcpp::export]]
-std::vector<std::vector<double>>  sbayes_reg( std::vector<double>& wy,
+std::vector<std::vector<double>>  sbayes_reg( double yy,
+                                              std::vector<double>& wy,
                                               std::vector<double>& ww, 
                                               std::vector<std::vector<double>>& LDvalues, 
                                               std::vector<std::vector<int>>& LDindices, 
                                               std::vector<double> b, 
                                               std::vector<double> lambda, 
                                               std::vector<bool> mask, 
-                                              double yy, 
                                               std::vector<double> pi, 
                                               std::vector<double> gamma, 
                                               double vb, 
@@ -1397,9 +1100,9 @@ std::vector<std::vector<double>>  sbayes_reg( std::vector<double>& wy,
                                               double nug,
                                               double nue,
                                               bool updateB,
+                                              bool updateG,
                                               bool updateE,
                                               bool updatePi,
-                                              bool updateG,
                                               int n, 
                                               int nit,
                                               int nburn,
@@ -1760,9 +1463,9 @@ std::vector<std::vector<double>>  sbayes_reg_eigen( std::vector<double>& wy,
                                                     double nug,
                                                     double nue,
                                                     bool updateB,
+                                                    bool updateG,
                                                     bool updateE,
                                                     bool updatePi,
-                                                    bool updateG,
                                                     int n, 
                                                     int nit,
                                                     int nburn,
@@ -2014,3 +1717,268 @@ std::vector<std::vector<double>>  sbayes_reg_eigen( std::vector<double>& wy,
   return result;
 }
 
+// // [[Rcpp::export]]
+// std::vector<std::vector<double>>  sbayes( std::vector<double> wy,
+//                                           std::vector<std::vector<double>> LD,
+//                                           std::vector<double> b,
+//                                           std::vector<double> lambda,
+//                                           double yy,
+//                                           double pi,
+//                                           double vg,
+//                                           double vb,
+//                                           double ve,
+//                                           double ssb_prior,
+//                                           double sse_prior,
+//                                           double nub,
+//                                           double nue,
+//                                           bool updateB,
+//                                           bool updateE,
+//                                           bool updatePi,
+//                                           int n,
+//                                           int nit,
+//                                           int method) {
+//   
+//   // Define local variables
+//   int m = b.size();
+//   
+//   double rhs, lhs, bn, conv, diff;
+//   double rhs0, rhs1, lhs0, lhs1, like0, like1, p0;
+//   double ssb, sse, ssg, dfb, dfe, chi2;
+//   double xtau, tau, lambda_tau, mu_tau, z, z2, u;
+//   
+//   std::vector<int> d(m);
+//   
+//   std::vector<double> ww(m),r(m);
+//   
+//   std::vector<double> dm(m),bm(m);
+//   std::vector<double> ves(nit),vbs(nit),pis(nit);
+//   
+//   std::vector<double> x2(m);
+//   std::vector<int> order(m);
+//   
+//   // Initialize variables
+//   for ( int i = 0; i < m; i++) {
+//     dm[i] = 0.0;
+//     bm[i] = 0.0;
+//     ww[i] = LD[i][i];
+//     r[i] = wy[i];
+//     x2[i] = (wy[i]/ww[i])*(wy[i]/ww[i]);
+//   }
+//   
+//   for (int i=0; i < nit; i++) {
+//     vbs[i] = 0.0;
+//     ves[i] = 0.0;
+//     pis[i] = 0.0;
+//   }
+//   
+//   // Establish order of markers as they are entered into the model
+//   std::iota(order.begin(), order.end(), 0);
+//   std::sort(  std::begin(order),
+//               std::end(order),
+//               [&](int i1, int i2) { return x2[i1] > x2[i2]; } );
+//   
+//   // Wy - W'Wb
+//   for ( int i = 0; i < m; i++) {
+//     if (b[i]!= 0.0) {
+//       for (int j = 0; j < m; j++) {
+//         r[j]=r[j] - LD[i][j]*b[i];
+//       }
+//     }
+//   }
+//   
+//   // Start Gibbs sampler
+//   std::random_device rd;
+//   unsigned int local_seed;
+//   local_seed = rd();
+//   std::mt19937 gen(local_seed);
+//   
+//   for ( int it = 0; it < nit; it++) {
+//     conv = 0.0;
+//     
+//     // Compute marker effects (BLUP)
+//     if (method==0) {
+//       for ( int isort = 0; isort < m; isort++) {
+//         int i = order[isort];
+//         lhs = ww[i] + lambda[i];
+//         rhs = r[i] + ww[i]*b[i];
+//         bn = rhs/lhs;
+//         diff = bn-b[i];
+//         for (int j = 0; j < m; j++) {
+//           r[j]=r[j] - LD[i][j]*(diff);
+//         }
+//         conv = conv + diff*diff;
+//         b[i] = bn;
+//       }
+//     }
+//     
+//     // Sample marker effects (Mixed, BayesA, Lasso)
+//     if ( method==1 || method==2 || method==3 ) {
+//       for ( int isort = 0; isort < m; isort++) {
+//         int i = order[isort];
+//         lhs = ww[i] + lambda[i];
+//         rhs = r[i] + ww[i]*b[i];
+//         std::normal_distribution<double> rnorm(rhs/lhs, sqrt(ve/lhs));
+//         bn = rnorm(gen);
+//         diff = bn-b[i];
+//         for (int j = 0; j < m; j++) {
+//           r[j]=r[j] - LD[i][j]*(diff);
+//         }
+//         conv = conv + diff*diff;
+//         b[i] = bn;
+//       }
+//     }
+//     
+//     // Sample marker effects (BayesC)
+//     if (method==4) {
+//       for ( int isort = 0; isort < m; isort++) {
+//         int i = order[isort];
+//         lhs0 = 1.0/vb;
+//         lhs1 = ww[i]/ve + 1.0/vb;
+//         rhs0 = 0.0;
+//         rhs1 = 0.0;
+//         rhs1 = r[i]/ve + ww[i]*b[i]/ve;
+//         like0 = sqrt((1.0/lhs0))*std::exp(0.5*(1.0/lhs0)*rhs0*rhs0);
+//         like1 = sqrt((1.0/lhs1))*std::exp(0.5*(1.0/lhs1)*rhs1*rhs1);
+//         like0 = like0*(1.0-pi);
+//         like1 = like1*pi;
+//         p0 = like0/(like0+like1);
+//         d[i]=0;
+//         std::uniform_real_distribution<double> runif(0.0, 1.0);
+//         u = runif(gen);
+//         if(u>p0) d[i]=1;
+//         bn=0.0;
+//         if(d[i]==1) {
+//           std::normal_distribution<double> rnorm(rhs1/lhs1, sqrt(1.0/lhs1));
+//           bn = rnorm(gen);
+//         }
+//         diff = bn-b[i];
+//         if(diff!=0.0) {
+//           for (int j = 0; j < m; j++) {
+//             r[j]=r[j] - LD[i][j]*(diff);
+//           }
+//           conv = conv + diff*diff;
+//         }
+//         b[i] = bn;
+//       }
+//     }
+//     
+//     // Sample marker variance
+//     ssb = 0.0;
+//     dfb = 0.0;
+//     for ( int i = 0; i < m; i++) {
+//       bm[i] = bm[i] + b[i];
+//       if(d[i]==1)   {
+//         ssb = ssb + b[i]*b[i];
+//         dfb = dfb + 1.0;
+//         dm[i] = dm[i] + 1.0;
+//       }
+//     }
+//     if(updateB) {
+//       std::chi_squared_distribution<double> rchisq(dfb+nub);
+//       chi2 = rchisq(gen);
+//       vb = (ssb + ssb_prior*nub)/chi2 ;
+//       vbs[it] = vb; 
+//     }
+//     
+//     
+//     // Sample residual variance
+//     if(updateE) {
+//       dfe = (double)n + nue;
+//       ssg = 0.0;
+//       sse = 0.0;
+//       for ( int i = 0; i < m; i++) {
+//         ssg = ssg + b[i] * (wy[i] -  r[i]);
+//         sse = sse + b[i] * (r[i] + wy[i]);
+//       }
+//       sse = yy - sse;
+//       std::chi_squared_distribution<double> rchisq(dfe);
+//       chi2 = rchisq(gen);
+//       ve = (sse + sse_prior*nue)/chi2 ;
+//       ves[it] = ve;
+//     }
+//     
+//     // Update lambda's for BLUP/Mixed
+//     if ( method==1) {
+//       for ( int i = 0; i < m; i++) {
+//         lambda[i] = ve/vb;
+//       }
+//     }
+//     
+//     // Sample marker specific variance for Bayes A
+//     if (method==2) {
+//       dfb = 1.0 + nub;
+//       for ( int i = 0; i < m; i++) {
+//         ssb = b[i]*b[i];
+//         std::chi_squared_distribution<double> rchisq(dfb);
+//         chi2 = rchisq(gen);
+//         vb = (ssb + ssb_prior*nub)/chi2 ;
+//         lambda[i] = ve/vb;
+//       }
+//     }
+//     
+//     // Sample marker specific tau for Bayesian lasso
+//     if (method==3) {
+//       lambda_tau = 2.0*2.0*0.5*0.5/vb;
+//       for ( int i = 0; i < m; i++) {
+//         ssb = b[i]*b[i];
+//         mu_tau = sqrt(lambda_tau/ssb);
+//         std::normal_distribution<double> norm(0.0, 1.0);
+//         z = norm(gen);
+//         z2=z*z;
+//         xtau=mu_tau+0.5*mu_tau*mu_tau*z2/lambda_tau - 0.5*(mu_tau/lambda_tau)*sqrt(4*mu_tau*lambda_tau*z2+mu_tau*mu_tau*z2*z2);
+//         std::uniform_real_distribution<double> runif(0.0, 1.0);
+//         u = runif(gen);
+//         tau = mu_tau*mu_tau/xtau;
+//         if(u <= mu_tau/(mu_tau+xtau)) tau=xtau;
+//         lambda[i] = ve/tau;
+//       }
+//     }
+//     
+//     // Sample pi for Bayes C
+//     if(method==4 && updatePi) {
+//       double count = dfb + 1.0;
+//       std::gamma_distribution<double> rgamma(count,1.0);
+//       double rg = rgamma(gen);
+//       pi = rg/(double)m;
+//       pis[it] = pi;
+//     }
+//   }
+//   
+//   // Summarize results
+//   std::vector<std::vector<double>> result(10);
+//   result[0].resize(m);
+//   result[1].resize(m);
+//   result[2].resize(nit);
+//   result[3].resize(nit);
+//   result[4].resize(nit);
+//   result[5].resize(nit);
+//   result[6].resize(m);
+//   result[7].resize(m);
+//   result[8].resize(3);
+//   result[9].resize(m);
+//   
+//   for (int i=0; i < m; i++) {
+//     result[0][i] = bm[i]/nit;
+//     result[1][i] = dm[i]/nit;
+//   }
+//   for (int i=0; i < nit; i++) {
+//     //result[2][i] = mus[i];
+//     result[2][i] = 0.0;
+//     result[3][i] = vbs[i];
+//     result[4][i] = ves[i];
+//     result[5][i] = pis[i];
+//   }
+//   for (int i=0; i < m; i++) {
+//     result[6][i] = wy[i];
+//     result[7][i] = r[i];
+//   }
+//   result[8][0] = vb;
+//   result[8][1] = ve;
+//   result[8][2] = pi;
+//   for (int i=0; i < m; i++) {
+//     result[9][i] = b[i];
+//   }
+//   
+//   
+//   return result;
+// }
